#include "stdafx.h"
#include "common.h"


namespace cmmn {

	using namespace System;

	int n;
	double * st;
	int * resp;

	const int NPoints = 501;

	double ** gprob;
	double * probPSE, * probSGM;

	double minPSE, maxPSE, minJND, maxJND, minSGM, maxSGM, k_width;

	const double coeff = 0.674489750;  //   = z(0.75)


	double sqr( double x ){
			return x * x;
	}


	/**************************************************************************

		  Kerridge and Cook(1976, Biometrika, 63, 401-403)'s Algorithm
							  referred to
		   in  A. L. Brophy and  D. L. Wood (1989, Behav. Res. Meth. Istr.
											 & Comp., 21, 447-454)

	**************************************************************************/

	double normal_0_to_z( double z ) {
		double p;
		z = Math::Abs(z);
		if (z > 6.8) {
			double zz = z * z;
			double d = zz + 3 - 1/(0.22 * zz + 0.704);
			double m = 1 - 1/d;
			p = 0.5 - Math::Exp(-0.5 * zz) * m / (z * Math::Sqrt(2 * Math::PI));
		} else {
			p = 0.0;
			if (z > 0.0) {
				double zz4 = 0.25 * z * z;
				double n2 = 2.0;                // initial value of 2n          ; n = 1
				double theta2 = 1.0;			// initial value of theta(2n-2)
				double theta1 = zz4;			// initial value of theta(2n-1)
				p = 1.0;						// sum for n = 0
				double p_prev = 0.0;			// initial value of sub-sum
				while (true) {
					theta2 = zz4 * (theta1 - theta2) / n2;
					n2 += 1.0;
					p += theta2/n2;
					if (p == p_prev) break;
					theta1 = zz4 * (theta2 - theta1) / n2;
					n2 += 1.0;
					p_prev = p;
				}
				p = z * Math::Exp(-0.5 * zz4) * p / Math::Sqrt(2.0 * Math::PI);
			};
		}
		return p;
	}


	double Cum_NormalKC( double z ) {
		if ( z >= 0.0 ) {
			if (z > 0.0) {
				double v = 0.5 + normal_0_to_z( z );
				if (v > 1.0) {
					return 1.0;
				} else {
					return v;
				};
			} else {
				return 0.5;
			};
		} else {
			double v = 0.5 - normal_0_to_z( Math::Abs(z) );
			if (v < 0.0){
				return 0.0;
			} else {
				return v;
			};
		}
	}



	double PF( double x, double mu, double sgm ){
		double v = Cum_NormalKC( (x - mu) / sgm );
		return v;
	}


	void calcPArrayMed( double * p, int n /* 0..n */, int & ip0, int & ip1, double & w0, double & w1  ){
		ip0 = 0;
		double sum = 0.0;
		do {
			sum += p[ip0++];
		} while (sum < 0.5);
		ip1 = --ip0;
		w0 = sum - 0.5;
		ip0 = ip1 - 1;
		w1 = 0.5 - (sum - p[ip1]);
		return;
	}

	void calcPArrayMode( double * p, int n /* 0..n */, int & ip ){
		ip = 0;
		for (int i = 1; i <= n; i++)
			if (p[ip] < p[i]) ip = i;
		return;
	}



	void calcPArrayHPD( double * p, int n, double confLevel, int & iL0, int & iL1, int & iR0, int & iR1  ){
			 double ckIntvl = 1.0;
			 double pAlpha = 1.0 - confLevel;
			 iL0 = 0; iR0 = n;
			 int ipos = 0;
			 double AlphaL = p[ipos];
			 while (AlphaL <= pAlpha) {
				int iposR = n;
				double AlphaR = p[iposR];
				while (AlphaR < (pAlpha - AlphaL)) AlphaR += p[--iposR];
				if (AlphaR > (pAlpha - AlphaL)) iposR++;
				if ((iposR - ipos) < (iR0 - iL0)){
					iL0 = ipos;  iR0 = iposR;
				}
				AlphaL += p[++ipos];
			 }

			 iL1 = 0; iR1 = n;
			 ipos = n;
			 double AlphaR = p[ipos];
			 while (AlphaR <= pAlpha) {
				int iposL = 0;
				double AlphaL = p[iposL];
				while (AlphaL < (pAlpha - AlphaR)) AlphaL += p[++iposL];
				if (AlphaL > (pAlpha - AlphaR)) iposL--;
				if ((ipos - iposL) < (iR1 - iL1)){
					iL1 = iposL;  iR1 = ipos;
				}
				AlphaR += p[--ipos];
			 }

			 return;
	}


	double kernel( double x, double c, double w ){
		return Math::Exp(-0.5 * sqr((x - c) / w));
	}


	double k_ave( double x, int * vy, double * vx, int n, double w ){
		double v = 0.0, sw = 0.0;
		for (int i = 0; i < n; i++){
			double kv = kernel( x, vx[i], w );
			v += vy[i] * kv;
			sw += kv;
		}
		if (sw > 0.0) v = v / sw;
		else		  v = 0.0;
		return v;
	}


	//
	//		HPĎvZ
	//

	double CalcHPD::contDistri(double x){
		double v;
		if (x <= 0.0) {
					v = p[0] * (x + 1.0);
		} else if (x >= n){
					v = p[n] * (n + 1.0 - x);
		} else {
					int Lx = int(x);
					int Ux = Lx + 1;
					double w1 = x - Lx;
					double w2 = Ux - x;
					v = p[Lx] * w2 + p[Ux] * w1;
		}

		return v;
	}

	double CalcHPD::cumDistri( double x ){
		double v = 0.0;
		if (x <= 0.0){
				v = 0.5 * p[0] * sqr(x + 1.0);
		}else if (x >= n){
				for (int i = 0; i < n; i++)
					v += p[i];
				v += 0.5 * p[n] + 0.5 * p[n] * (1.0 - sqr(n + 1.0 - x));
		}else{
				int Lx = int(x);
				if (Lx > 0)
					for (int i = 0; i < Lx; i++)
						v += p[i];
				v += 0.5 * p[Lx] + 0.5 * (contDistri(Lx) + contDistri(x)) * (x - Lx);
		}

		return v;
	}

	double CalcHPD::Useek_point( double cum_p, double Lx, double Ux, double (*f)(double v) ){
		if ((f(Lx) >= cum_p) || (f(Ux) <= cum_p)){
				throw gcnew Exception("Error in Dssek_point\n" +
					"f(Lx) = " + f(Lx).ToString() + "  cum_p = " + cum_p.ToString() + "  f(Ux) = " + f(Ux).ToString() + "\n");
		}

		double m = 0.5 * (Lx + Ux);
		do{
			if (f(m) < cum_p){
					Lx = m;
			}else{
					Ux = m;
			}
			m = 0.5 * (Lx + Ux);
		} while (Math::Abs(Ux - Lx) > 1.0e-9);

		return m;
	}

	double CalcHPD::Dseek_point( double cum_p, double Lx, double Ux, double (*f)(double v) ){

		double m = 0.5 * (Lx + Ux);
		do{
			if (f(m) > cum_p){
					Lx = m;
			}else{
					Ux = m;
			}
			m = 0.5 * (Lx + Ux);
		} while (Math::Abs(Ux - Lx) > 1.0e-9);

		return m;
	}

	CalcHPD *myC;
	double FcontDistri( double x ){
		double v = myC->contDistri(x);
		return v;
	}

	double CalcHPD::CalcIntvl( double h ){
		double v;
		if (h >= maxH){
				Lpoint = Rpoint = 1.0e100;
				v = 1.0;
		}else if (h <= 0.0){
				Lpoint = Rpoint = -1.0e100;
				v = 0.0;
		}else{
			myC = this;
			Lpoint = Useek_point( h, 0.0, mode, FcontDistri );
			Rpoint = Dseek_point( h, mode, n, FcontDistri );
				v = cumDistri(Lpoint) + (1.0 - cumDistri(Rpoint));
		}

		return v;
	}

	double FCalcIntvl( double h ){
		double v = myC->CalcIntvl( h );
		return v;
	}

	void CalcHPD::Seek95Intvl( ){
		double LH = maxH * 0.1;
		while (CalcIntvl(LH) >= 0.05) {
			System::Diagnostics::Debug::WriteLine("CalcInvl = " + CalcIntvl(LH).ToString() + "   LH = " + LH.ToString());
				LH *= 0.5;
		}
		System::Diagnostics::Debug::WriteLine("LH = " + LH.ToString() + "   CalcIntvl = " + CalcIntvl(LH).ToString());
		double RH = maxH * 0.5;
		while (CalcIntvl(RH) <= 0.05) {
			System::Diagnostics::Debug::WriteLine("CalcInvl = " + CalcIntvl(LH).ToString() + "   RH = " + RH.ToString());
				RH += 0.5 * (maxH - RH);
		}
		System::Diagnostics::Debug::WriteLine("RH = " + RH.ToString() + "   CalcIntvl = " + CalcIntvl(RH).ToString());
		myC = this;
		Useek_point( 0.05, LH, RH, FCalcIntvl );
		/***
				Lpoint and Rpoint are set.
		***/
	}


}
